@@ -2,23 +2,23 @@
  * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
 /*
- * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GLib at ftp://ftp.gtk.org/pub/gtk/. 
@@ -190,7 +190,8 @@ g_log_domain_check_free (GLogDomain *domain)
 	      g_free (domain);
 	      break;
 	    }
-	  work = work->next;
+	  last = work;
+	  work = last->next;
 	}  
       g_mutex_unlock (g_messages_lock);
     }
@@ -328,7 +329,8 @@ g_log_remove_handler (const gchar *log_domain,
 	      g_log_domain_check_free (domain);
 	      return;
 	    }
-	  work = work->next;
+	  last = work;
+	  work = last->next;
 	}
     }
   g_warning ("g_log_remove_handler(): could not find handler with id `%d' for domain \"%s\"",
@@ -469,7 +471,7 @@ g_log_default_handler (const gchar    *log_domain,
    */
   fd = stdout;
 #else
-  fd = (log_level >= G_LOG_LEVEL_MESSAGE) ? 1 : 2;
+  fd = (log_level > G_LOG_LEVEL_MESSAGE) ? 1 : 2;
 #endif
   
   g_mutex_lock (g_messages_lock);
