@@ -18,6 +18,7 @@
  */
 #include	"gtype.h"
 
+#include	"gtypeplugin.h"
 #include	<string.h>
 
 #define FIXME_DISABLE_PREALLOCATIONS
@@ -32,20 +33,22 @@
  * TODO:
  * - g_type_from_name() should do an ordered array lookup after fetching the
  *   the quark, instead of a second hashtable lookup.
+ * - speedup checks for virtual types, steal a bit somewhere
  *
  * FIXME:
  * - force interface initialization for already existing classes
+ * - make things threadsafe
  */
 
-#define G_TYPE_FLAG_MASK	(G_TYPE_FLAG_CLASSED | \
-				 G_TYPE_FLAG_INSTANTIATABLE | \
-				 G_TYPE_FLAG_DERIVABLE | \
-				 G_TYPE_FLAG_DEEP_DERIVABLE)
-#define	g_type_plugin_ref(p)				((p)->vtable->plugin_ref (p))
-#define g_type_plugin_unref(p)				((p)->vtable->plugin_unref (p))
-#define	g_type_plugin_complete_type_info(p,t,i,v)	((p)->vtable->complete_type_info ((p), (t), (i), (v)))
-#define	g_type_plugin_complete_interface_info(p,f,t,i)	((p)->vtable->complete_interface_info ((p), (f), (t), (i)))
+#define TYPE_FUNDAMENTAL_FLAG_MASK (G_TYPE_FLAG_CLASSED | \
+				    G_TYPE_FLAG_INSTANTIATABLE | \
+				    G_TYPE_FLAG_DERIVABLE | \
+				    G_TYPE_FLAG_DEEP_DERIVABLE)
+#define	TYPE_FLAG_MASK		   (G_TYPE_FLAG_ABSTRACT)
 
+
+
+/* --- typedefs --- */
 typedef struct _TypeNode        TypeNode;
 typedef struct _CommonData      CommonData;
 typedef struct _IFaceData       IFaceData;
@@ -58,6 +61,8 @@ typedef struct _IFaceHolder	IFaceHolder;
 
 /* --- prototypes --- */
 static inline GTypeFundamentalInfo*	type_node_fundamental_info	(TypeNode		*node);
+static	      void			type_add_flags			(TypeNode		*node,
+									 GTypeFlags		 flags);
 static	      void			type_data_make			(TypeNode        	*node,
 									 const GTypeInfo	*info,
 									 const GTypeValueTable	*value_table);
@@ -69,6 +74,10 @@ static	      void			type_data_last_unref		(GType			 type,
 
 
 /* --- structures --- */
+struct _GValue	/* kludge, keep in sync with gvalue.h */
+{
+  GType g_type;
+};
 struct _TypeNode
 {
   GTypePlugin *plugin;
@@ -159,11 +168,12 @@ typedef struct {
 /* --- variables --- */
 static guint           n_class_cache_funcs = 0;
 static ClassCacheFunc *class_cache_funcs = NULL;
+static GType           last_fundamental_id = 0;
+static GQuark          quark_type_flags = 0;
 
 
 /* --- externs --- */
-const char *g_log_domain_gobject = "GLib-Object";
-GOBJECT_VAR GType _g_type_fundamental_last = 0;
+const char  *g_log_domain_gobject = "GLib-Object";
 
 
 /* --- type nodes --- */
@@ -177,7 +187,7 @@ LOOKUP_TYPE_NODE (register GType utype)
   register GType ftype = G_TYPE_FUNDAMENTAL (utype);
   register GType b_seqno = G_TYPE_BRANCH_SEQNO (utype);
 
-  if (ftype < G_TYPE_FUNDAMENTAL_LAST && b_seqno < g_branch_seqnos[ftype])
+  if (ftype < last_fundamental_id && b_seqno < g_branch_seqnos[ftype])
     return g_type_nodes[ftype][b_seqno];
   else
     return NULL;
@@ -187,11 +197,11 @@ LOOKUP_TYPE_NODE (register GType utype)
 #define NODE_NAME(node)         (g_quark_to_string (node->qname))
 
 static TypeNode*
-type_node_any_new (TypeNode    *pnode,
-		   GType        ftype,
-		   const gchar *name,
-		   GTypePlugin *plugin,
-		   GTypeFlags   type_flags)
+type_node_any_new (TypeNode              *pnode,
+		   GType                  ftype,
+		   const gchar           *name,
+		   GTypePlugin           *plugin,
+		   GTypeFundamentalFlags  type_flags)
 {
   guint branch_last, n_supers = pnode ? pnode->n_supers + 1 : 0;
   GType type;
@@ -204,11 +214,11 @@ type_node_any_new (TypeNode    *pnode,
     g_type_nodes[ftype] = g_renew (TypeNode*, g_type_nodes[ftype], 1 << g_bit_storage (g_branch_seqnos[ftype] - 1));
 
   if (!pnode)
-    node_size += sizeof (GTypeFundamentalInfo);	 /* fundamental type info */
-  node_size += SIZEOF_BASE_TYPE_NODE ();	 /* TypeNode structure */
+    node_size += sizeof (GTypeFundamentalInfo);	      /* fundamental type info */
+  node_size += SIZEOF_BASE_TYPE_NODE ();	      /* TypeNode structure */
   node_size += (sizeof (GType) * (1 + n_supers + 1)); /* self + ancestors + 0 for ->supers[] */
   node = g_malloc0 (node_size);
-  if (!pnode)					 /* fundamental type */
+  if (!pnode)					      /* offset fundamental types */
     node = G_STRUCT_MEMBER_P (node, sizeof (GTypeFundamentalInfo));
   g_type_nodes[ftype][branch_last] = node;
 
@@ -276,28 +286,28 @@ type_node_fundamental_info (TypeNode *node)
   if (ftype != NODE_TYPE (node))
     node = LOOKUP_TYPE_NODE (ftype);
   
-  return node ? G_STRUCT_MEMBER_P (node, - sizeof (GTypeFundamentalInfo)) : NULL;
+  return node ? G_STRUCT_MEMBER_P (node, - (gssize) sizeof (GTypeFundamentalInfo)) : NULL;
 }
 
 static TypeNode*
-type_node_fundamental_new (GType        ftype,
-			   const gchar *name,
-			   GTypeFlags   type_flags)
+type_node_fundamental_new (GType                 ftype,
+			   const gchar          *name,
+			   GTypeFundamentalFlags type_flags)
 {
   GTypeFundamentalInfo *finfo;
   TypeNode *node;
-  guint i, flast = G_TYPE_FUNDAMENTAL_LAST;
+  guint i, flast = last_fundamental_id;
   
   g_assert (ftype == G_TYPE_FUNDAMENTAL (ftype));
   
-  type_flags &= G_TYPE_FLAG_MASK;
+  type_flags &= TYPE_FUNDAMENTAL_FLAG_MASK;
 
-  _g_type_fundamental_last = MAX (_g_type_fundamental_last, ftype + 1);
-  if (G_TYPE_FUNDAMENTAL_LAST > flast)
+  last_fundamental_id = MAX (last_fundamental_id, ftype + 1);
+  if (last_fundamental_id > flast)
     {
-      g_type_nodes = g_renew (TypeNode**, g_type_nodes, G_TYPE_FUNDAMENTAL_LAST);
-      g_branch_seqnos = g_renew (GType, g_branch_seqnos, G_TYPE_FUNDAMENTAL_LAST);
-      for (i = flast; i < G_TYPE_FUNDAMENTAL_LAST; i++)
+      g_type_nodes = g_renew (TypeNode**, g_type_nodes, last_fundamental_id);
+      g_branch_seqnos = g_renew (GType, g_branch_seqnos, last_fundamental_id);
+      for (i = flast; i < last_fundamental_id; i++)
 	{
 	  g_type_nodes[i] = NULL;
 	  g_branch_seqnos[i] = 0;
@@ -385,31 +395,19 @@ check_plugin (GTypePlugin *plugin,
 		 type_name);
       return FALSE;
     }
-  if (!plugin->vtable)
+  if (!G_IS_TYPE_PLUGIN (plugin))
     {
-      g_warning ("plugin for type `%s' has no function table",
-		 type_name);
+      g_warning ("plugin pointer (%p) for type `%s' is invalid",
+		 plugin, type_name);
       return FALSE;
     }
-  if (!plugin->vtable->plugin_ref)
-    {
-      g_warning ("plugin for type `%s' has no plugin_ref() implementation",
-		 type_name);
-      return FALSE;
-    }
-  if (!plugin->vtable->plugin_unref)
-    {
-      g_warning ("plugin for type `%s' has no plugin_unref() implementation",
-		 type_name);
-      return FALSE;
-    }
-  if (need_complete_type_info && !plugin->vtable->complete_type_info)
+  if (need_complete_type_info && !G_TYPE_PLUGIN_GET_CLASS (plugin)->complete_type_info)
     {
       g_warning ("plugin for type `%s' has no complete_type_info() implementation",
 		 type_name);
       return FALSE;
     }
-  if (need_complete_interface_info && !plugin->vtable->complete_interface_info)
+  if (need_complete_interface_info && !G_TYPE_PLUGIN_GET_CLASS (plugin)->complete_interface_info)
     {
       g_warning ("plugin for type `%s' has no complete_interface_info() implementation",
 		 type_name);
@@ -495,6 +493,7 @@ check_value_table (const gchar           *type_name,
   else if (value_table->value_init == NULL)
     {
       if (value_table->value_free || value_table->value_copy ||
+	  value_table->value_peek_pointer ||
 	  value_table->collect_type || value_table->collect_value ||
 	  value_table->lcopy_type || value_table->lcopy_value)
 	g_warning ("cannot handle uninitializable values of type `%s'",
@@ -695,6 +694,25 @@ check_interface_info (TypeNode             *iface,
 
 
 /* --- type info (type node data) --- */
+static void
+type_add_flags (TypeNode  *node,
+		GTypeFlags flags)
+{
+  guint dflags;
+
+  g_return_if_fail ((flags & ~TYPE_FLAG_MASK) == 0);
+  g_return_if_fail (node != NULL);
+
+  if (!quark_type_flags)
+    quark_type_flags = g_quark_from_static_string ("GTypeFlags");
+  if ((flags & G_TYPE_FLAG_ABSTRACT) && node->is_classed &&
+      node->data && node->data->class.class)
+    g_warning ("tagging type `%s' as abstract after class initialization", NODE_NAME (node));
+  dflags = GPOINTER_TO_UINT (g_type_get_qdata (NODE_TYPE (node), quark_type_flags));
+  dflags |= flags;
+  g_type_set_qdata (NODE_TYPE (node), quark_type_flags, GUINT_TO_POINTER (dflags));
+}
+
 static void
 type_data_make (TypeNode              *node,
 		const GTypeInfo       *info,
@@ -778,7 +796,7 @@ type_data_make (TypeNode              *node,
     *vtable = *value_table;
   node->data->common.value_table = vtable;
 
-  g_assert (node->data->common.value_table != NULL); // FIXME: paranoid
+  g_assert (node->data->common.value_table != NULL); /* FIXME: paranoid */
 }
 
 static inline void
@@ -797,7 +815,7 @@ type_data_ref (TypeNode *node)
       
       memset (&tmp_info, 0, sizeof (tmp_info));
       memset (&tmp_value_table, 0, sizeof (tmp_value_table));
-      g_type_plugin_ref (node->plugin);
+      g_type_plugin_use (node->plugin);
       g_type_plugin_complete_type_info (node->plugin, NODE_TYPE (node), &tmp_info, &tmp_value_table);
       check_type_info (pnode, G_TYPE_FUNDAMENTAL (NODE_TYPE (node)), NODE_NAME (node), &tmp_info);
       type_data_make (node, &tmp_info,
@@ -856,10 +874,10 @@ type_node_add_iface_entry (TypeNode *node,
 }
 
 static void
-type_add_interface (TypeNode       *node,
-		    TypeNode       *iface,
-		    GInterfaceInfo *info,
-		    GTypePlugin    *plugin)
+type_add_interface (TypeNode             *node,
+		    TypeNode             *iface,
+		    const GInterfaceInfo *info,
+		    GTypePlugin          *plugin)
 {
   IFaceHolder *iholder = g_new0 (IFaceHolder, 1);
   
@@ -897,7 +915,7 @@ type_iface_retrive_holder_info (TypeNode *iface,
       type_data_ref (iface);
 
       memset (&tmp_info, 0, sizeof (tmp_info));
-      g_type_plugin_ref (iholder->plugin);
+      g_type_plugin_use (iholder->plugin);
       g_type_plugin_complete_interface_info (iholder->plugin, NODE_TYPE (iface), instance_type, &tmp_info);
       check_interface_info (iface, instance_type, &tmp_info);
       iholder->info = g_memdup (&tmp_info, sizeof (tmp_info));
@@ -921,7 +939,7 @@ type_iface_blow_holder_info (TypeNode *iface,
     {
       g_free (iholder->info);
       iholder->info = NULL;
-      g_type_plugin_unref (iholder->plugin);
+      g_type_plugin_unuse (iholder->plugin);
 
       type_data_unref (iface, FALSE);
     }
@@ -943,6 +961,12 @@ g_type_create_instance (GType type)
 		 type_descriptive_name (type));
       return NULL;
     }
+  if (G_TYPE_IS_ABSTRACT (type))
+    {
+      g_warning ("cannot create instance of abstract (non-instantiatable) type `%s'",
+		 type_descriptive_name (type));
+      return NULL;
+    }
   
   class = g_type_class_ref (type);
   
@@ -992,8 +1016,15 @@ g_type_free_instance (GTypeInstance *instance)
 		 type_descriptive_name (class->g_type));
       return;
     }
+  if (G_TYPE_IS_ABSTRACT (NODE_TYPE (node)))
+    {
+      g_warning ("cannot free instance of abstract (non-instantiatable) type `%s'",
+		 NODE_NAME (node));
+      return;
+    }
 
   instance->g_class = NULL;
+  memset (instance, 0xaa, node->data->instance.instance_size);	// FIXME
   if (node->data->instance.n_preallocs)
     g_chunk_free (instance, node->data->instance.mem_chunk);
   else
@@ -1098,10 +1129,10 @@ type_class_init (TypeNode   *node,
    */
   for (bnode = node; bnode; bnode = LOOKUP_TYPE_NODE (NODE_PARENT_TYPE (bnode)))
     if (bnode->data->class.class_init_base)
-      init_slist = g_slist_prepend (init_slist, bnode->data->class.class_init_base);
+      init_slist = g_slist_prepend (init_slist, (gpointer) bnode->data->class.class_init_base);
   for (slist = init_slist; slist; slist = slist->next)
     {
-      GBaseInitFunc class_init_base = slist->data;
+      GBaseInitFunc class_init_base = (GBaseInitFunc) slist->data;
       
       class_init_base (class);
     }
@@ -1233,7 +1264,7 @@ type_data_last_unref (GType    type,
       
       if (ptype)
 	type_data_unref (LOOKUP_TYPE_NODE (ptype), FALSE);
-      g_type_plugin_unref (node->plugin);
+      g_type_plugin_unuse (node->plugin);
     }
 }
 
@@ -1282,7 +1313,8 @@ GType
 g_type_register_fundamental (GType                       type_id,
 			     const gchar                *type_name,
 			     const GTypeInfo            *info,
-			     const GTypeFundamentalInfo *finfo)
+			     const GTypeFundamentalInfo *finfo,
+			     GTypeFlags			 flags)
 {
   GTypeFundamentalInfo *node_finfo;
   TypeNode *node;
@@ -1318,6 +1350,7 @@ g_type_register_fundamental (GType                       type_id,
 
   node = type_node_fundamental_new (type_id, type_name, finfo->type_flags);
   node_finfo = type_node_fundamental_info (node);
+  type_add_flags (node, flags);
 
   if (!check_type_info (NULL, G_TYPE_FUNDAMENTAL (NODE_TYPE (node)), type_name, info))
     return NODE_TYPE (node);
@@ -1330,7 +1363,8 @@ g_type_register_fundamental (GType                       type_id,
 GType
 g_type_register_static (GType            parent_type,
 			const gchar     *type_name,
-			const GTypeInfo *info)
+			const GTypeInfo *info,
+			GTypeFlags	 flags)
 {
   TypeNode *pnode, *node;
   GType type;
@@ -1351,12 +1385,13 @@ g_type_register_static (GType            parent_type,
     return 0;
   if (info->class_finalize)
     {
-      g_warning ("class destructor specified for static type `%s'",
+      g_warning ("class finalizer specified for static type `%s'",
 		 type_name);
       return 0;
     }
 
   node = type_node_new (pnode, type_name, NULL);
+  type_add_flags (node, flags);
   type = NODE_TYPE (node);
   type_data_make (node, info,
 		  check_value_table (type_name, info->value_table) ? info->value_table : NULL);
@@ -1367,7 +1402,8 @@ g_type_register_static (GType            parent_type,
 GType
 g_type_register_dynamic (GType        parent_type,
 			 const gchar *type_name,
-			 GTypePlugin *plugin)
+			 GTypePlugin *plugin,
+			 GTypeFlags   flags)
 {
   TypeNode *pnode, *node;
   GType type;
@@ -1385,15 +1421,16 @@ g_type_register_dynamic (GType        parent_type,
   pnode = LOOKUP_TYPE_NODE (parent_type);
 
   node = type_node_new (pnode, type_name, plugin);
+  type_add_flags (node, flags);
   type = NODE_TYPE (node);
 
   return type;
 }
 
 void
-g_type_add_interface_static (GType           instance_type,
-			     GType           interface_type,
-			     GInterfaceInfo *info)
+g_type_add_interface_static (GType                 instance_type,
+			     GType                 interface_type,
+			     const GInterfaceInfo *info)
 {
   TypeNode *node;
   TypeNode *iface;
@@ -1674,11 +1711,7 @@ g_type_conforms_to (GType type,
 	}
     }
   else
-    {
-      TypeNode *node = LOOKUP_TYPE_NODE (type);
-
-      return node && (node->is_iface || node->is_instantiatable);
-    }
+    return LOOKUP_TYPE_NODE (type) != NULL;
 
   return FALSE;
 }
@@ -1688,7 +1721,7 @@ g_type_fundamental_branch_last (GType type)
 {
   GType ftype = G_TYPE_FUNDAMENTAL (type);
 
-  return ftype < G_TYPE_FUNDAMENTAL_LAST ? g_branch_seqnos[ftype] : 0;
+  return ftype < last_fundamental_id ? g_branch_seqnos[ftype] : 0;
 }
 
 GType* /* free result */
@@ -1839,20 +1872,34 @@ g_type_set_qdata (GType    type,
 
 /* --- implementation details --- */
 gboolean
-g_type_check_flags (GType      type,
-		    GTypeFlags flags)
+g_type_check_flags (GType type,
+		    guint flags)
 {
   TypeNode *node = LOOKUP_TYPE_NODE (type);
-  
-  flags &= G_TYPE_FLAG_MASK;
+  gboolean result = FALSE;
+
   if (node)
     {
-      GTypeFundamentalInfo *finfo = type_node_fundamental_info (node);
+      guint fflags = flags & TYPE_FUNDAMENTAL_FLAG_MASK;
+      guint tflags = flags & TYPE_FLAG_MASK;
+
+      if (fflags)
+	{
+	  GTypeFundamentalInfo *finfo = type_node_fundamental_info (node);
+	  
+	  fflags = (finfo->type_flags & fflags) == fflags;
+	}
+      else
+	fflags = TRUE;
       
-      return (finfo->type_flags & flags) != 0;
+      if (tflags)
+	tflags = (tflags & GPOINTER_TO_UINT (g_type_get_qdata (type, quark_type_flags))) == tflags;
+      else
+	tflags = TRUE;
+      
+      result = tflags && fflags;
     }
-  
-  return FALSE;
+  return result;
 }
 
 GTypePlugin*
@@ -1863,11 +1910,44 @@ g_type_get_plugin (GType type)
   return node ? node->plugin : NULL;
 }
 
+GTypePlugin*
+g_type_interface_get_plugin (GType instance_type,
+			     GType interface_type)
+{
+  TypeNode *node = LOOKUP_TYPE_NODE (instance_type);  
+  TypeNode *iface = LOOKUP_TYPE_NODE (interface_type);
+  IFaceHolder *iholder;
+
+  g_return_val_if_fail (node == NULL, NULL);
+  g_return_val_if_fail (iface == NULL, NULL);
+  g_return_val_if_fail (G_TYPE_IS_INTERFACE (interface_type), NULL);
+  
+  iholder = iface->private.iface_conformants;
+  
+  while (iholder && iholder->instance_type != instance_type)
+    iholder = iholder->next;
+
+  if (!iholder)
+    {
+      g_warning (G_STRLOC ": Attempt to look up plugin for invalid instance/interface type pair.");
+      return NULL;
+    }
+
+  return iholder->plugin;
+}
+
+GType
+g_type_fundamental_last (void)
+{
+  return last_fundamental_id;
+}
+
 gboolean
 g_type_instance_conforms_to (GTypeInstance *type_instance,
 			     GType          iface_type)
 {
   return (type_instance && type_instance->g_class &&
+	  G_TYPE_IS_INSTANTIATABLE (type_instance->g_class->g_type) &&
 	  g_type_conforms_to (type_instance->g_class->g_type, iface_type));
 }
 
@@ -1875,7 +1955,35 @@ gboolean
 g_type_class_is_a (GTypeClass *type_class,
 		   GType       is_a_type)
 {
-  return (type_class && g_type_is_a (type_class->g_type, is_a_type));
+  return (type_class && G_TYPE_IS_CLASSED (type_class->g_type) &&
+	  g_type_is_a (type_class->g_type, is_a_type));
+}
+
+gboolean
+g_type_value_conforms_to (GValue *value,
+			  GType   type)
+{
+  TypeNode *node;
+  
+  if (!value)
+    return FALSE;
+  node = LOOKUP_TYPE_NODE (value->g_type);
+#if 0
+  if (!G_TYPE_IS_FUNDAMENTAL (value->g_type) && !node || !node->data)
+    node = LOOKUP_TYPE_NODE (G_TYPE_FUNDAMENTAL (value->g_type));
+#endif
+  if (!node || !node->data || node->data->common.ref_count < 1 ||
+      !node->data->common.value_table->value_init ||
+      !g_type_conforms_to (value->g_type, type))
+    return FALSE;
+  
+  return TRUE;
+}
+
+gboolean
+g_type_check_value (GValue *value)
+{
+  return value && g_type_value_conforms_to (value, value->g_type);
 }
 
 GTypeInstance*
@@ -1894,9 +2002,9 @@ g_type_check_instance_cast (GTypeInstance *type_instance,
 		 type_descriptive_name (iface_type));
       return type_instance;
     }
-  if (!G_TYPE_IS_CLASSED (type_instance->g_class->g_type))
+  if (!G_TYPE_IS_INSTANTIATABLE (type_instance->g_class->g_type))
     {
-      g_warning ("invalid unclassed type `%s' in cast to `%s'",
+      g_warning ("invalid uninstantiatable type `%s' in cast to `%s'",
 		 type_descriptive_name (type_instance->g_class->g_type),
 		 type_descriptive_name (iface_type));
       return type_instance;
@@ -1940,13 +2048,47 @@ g_type_check_class_cast (GTypeClass *type_class,
   return type_class;
 }
 
+gboolean
+g_type_check_instance (GTypeInstance *type_instance)
+{
+  /* this function is just here to make the signal system
+   * conveniently elaborated on instance checks
+   */
+  if (!type_instance)
+    {
+      g_warning ("instance is invalid (NULL) pointer");
+      return FALSE;
+    }
+  if (!type_instance->g_class)
+    {
+      g_warning ("instance with invalid (NULL) class pointer");
+      return FALSE;
+    }
+  if (!G_TYPE_IS_CLASSED (type_instance->g_class->g_type))
+    {
+      g_warning ("instance of invalid unclassed type `%s'",
+		 type_descriptive_name (type_instance->g_class->g_type));
+      return FALSE;
+    }
+  if (!G_TYPE_IS_INSTANTIATABLE (type_instance->g_class->g_type))
+    {
+      g_warning ("instance of invalid non-instantiatable type `%s'",
+		 type_descriptive_name (type_instance->g_class->g_type));
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
 
 /* --- foreign prototypes --- */
 extern void	g_value_types_init	(void); /* sync with gvaluetypes.c */
 extern void	g_enum_types_init	(void);	/* sync with genums.c */
 extern void     g_param_type_init       (void);	/* sync with gparam.c */
+extern void     g_boxed_type_init       (void);	/* sync with gboxed.c */
 extern void     g_object_type_init      (void);	/* sync with gobject.c */
 extern void	g_param_spec_types_init	(void);	/* sync with gparamspecs.c */
+extern void	g_signal_init		(void);	/* sync with gsignal.c */
 
 
 /* --- initialization --- */
@@ -1958,7 +2100,7 @@ g_type_init (void)
   TypeNode *node;
   GType type;
 
-  if (G_TYPE_FUNDAMENTAL_LAST)
+  if (last_fundamental_id)
     return;
 
   /* type qname hash table */
@@ -1966,10 +2108,10 @@ g_type_init (void)
 
   /* invalid type G_TYPE_INVALID (0)
    */
-  _g_type_fundamental_last = 1;
-  g_type_nodes = g_renew (TypeNode**, g_type_nodes, G_TYPE_FUNDAMENTAL_LAST);
+  last_fundamental_id = 1;
+  g_type_nodes = g_renew (TypeNode**, g_type_nodes, last_fundamental_id);
   g_type_nodes[0] = &type0_node;
-  g_branch_seqnos = g_renew (GType, g_branch_seqnos, G_TYPE_FUNDAMENTAL_LAST);
+  g_branch_seqnos = g_renew (GType, g_branch_seqnos, last_fundamental_id);
   g_branch_seqnos[0] = 1;
 
   /* void type G_TYPE_NONE
@@ -1983,9 +2125,13 @@ g_type_init (void)
   memset (&info, 0, sizeof (info));
   node = type_node_fundamental_new (G_TYPE_INTERFACE, "GInterface", G_TYPE_FLAG_DERIVABLE);
   type = NODE_TYPE (node);
-  type_data_make (node, &info, NULL); // FIXME
+  type_data_make (node, &info, NULL); /* FIXME */
   g_assert (type == G_TYPE_INTERFACE);
 
+  /* G_TYPE_TYPE_PLUGIN
+   */
+  g_type_plugin_get_type ();
+
   /* G_TYPE_* value types
    */
   g_value_types_init ();
@@ -1998,6 +2144,10 @@ g_type_init (void)
    */
   g_param_type_init ();
 
+  /* G_TYPE_PARAM
+   */
+  g_boxed_type_init ();
+
   /* G_TYPE_OBJECT
    */
   g_object_type_init ();
@@ -2005,4 +2155,8 @@ g_type_init (void)
   /* G_TYPE_PARAM_* pspec types
    */
   g_param_spec_types_init ();
+
+  /* Signal system
+   */
+  g_signal_init ();
 }
