@@ -5,23 +5,23 @@
  * Copyright (C) 1997, 1998 Tim Janik
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
 /*
- * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GLib at ftp://ftp.gtk.org/pub/gtk/. 
@@ -127,7 +127,7 @@ static inline
 GScannerKey*	g_scanner_lookup_internal (GScanner	*scanner,
 					   guint	 scope_id,
 					   const gchar	*symbol);
-static gint	g_scanner_key_equal	  (gconstpointer v1,
+static gboolean	g_scanner_key_equal	  (gconstpointer v1,
 					   gconstpointer v2);
 static guint	g_scanner_key_hash	  (gconstpointer v);
 static void	g_scanner_get_token_ll	  (GScanner	*scanner,
@@ -346,7 +346,7 @@ g_scanner_warn (GScanner       *scanner,
     }
 }
 
-static gint
+static gboolean
 g_scanner_key_equal (gconstpointer v1,
 		     gconstpointer v2)
 {
@@ -565,17 +565,21 @@ g_scanner_scope_foreach_symbol (GScanner       *scanner,
 void
 g_scanner_freeze_symbol_table (GScanner *scanner)
 {
-  g_return_if_fail (scanner != NULL);
-  
-  g_hash_table_freeze (scanner->symbol_table);
+#ifdef G_ENABLE_DEBUG
+  static gboolean first_call = TRUE;
+
+  if (first_call)
+    {
+      g_warning("g_scanner_freeze_symbol_table and "
+		"g_scanner_thaw_symbol_table are deprecated.");
+      first_call = FALSE;
+    }
+#endif /* G_ENABLE_DEBUG */
 }
 
 void
 g_scanner_thaw_symbol_table (GScanner *scanner)
 {
-  g_return_if_fail (scanner != NULL);
-  
-  g_hash_table_thaw (scanner->symbol_table);
 }
 
 GTokenType
