@@ -2,23 +2,23 @@
  * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
 /*
- * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GLib at ftp://ftp.gtk.org/pub/gtk/. 
@@ -797,26 +797,151 @@ extern const char * strsignal(int);
     }
 
   sprintf (msg, "unknown signal (%d)", signum);
-
+  
   return msg;
 }
 
+/* Functions g_strlcpy and g_strlcat were originally developed by
+ * Todd C. Miller <Todd.Miller@courtesan.com> to simplify writing secure code.
+ * See ftp://ftp.openbsd.org/pub/OpenBSD/src/lib/libc/string/strlcpy.3
+ * for more information.
+ */
+
+#ifdef HAVE_STRLCPY
+/* Use the native ones, if available; they might be implemented in assembly */
+gsize
+g_strlcpy (gchar       *dest,
+	   const gchar *src,
+	   gsize        dest_size)
+{
+  g_return_val_if_fail (dest != NULL, 0);
+  g_return_val_if_fail (src  != NULL, 0);
+  
+  return strlcpy (dest, src, dest_size);
+}
+
+gsize
+g_strlcat (gchar       *dest,
+	   const gchar *src,
+	   gsize        dest_size)
+{
+  g_return_val_if_fail (dest != NULL, 0);
+  g_return_val_if_fail (src  != NULL, 0);
+  
+  return strlcat (dest, src, dest_size);
+}
+
+#else /* ! HAVE_STRLCPY */
+/* g_strlcpy
+ *
+ * Copy string src to buffer dest (of buffer size dest_size).  At most
+ * dest_size-1 characters will be copied.  Always NUL terminates
+ * (unless dest_size == 0).  This function does NOT allocate memory.
+ * Unlike strncpy, this function doesn't pad dest (so it's often faster).
+ * Returns size of attempted result, strlen(src),
+ * so if retval >= dest_size, truncation occurred.
+ */
+gsize
+g_strlcpy (gchar       *dest,
+           const gchar *src,
+           gsize        dest_size)
+{
+  register gchar *d = dest;
+  register const gchar *s = src;
+  register gsize n = dest_size;
+  
+  g_return_val_if_fail (dest != NULL, 0);
+  g_return_val_if_fail (src  != NULL, 0);
+  
+  /* Copy as many bytes as will fit */
+  if (n != 0 && --n != 0)
+    do
+      {
+	register gchar c = *s++;
+	
+	*d++ = c;
+	if (c == 0)
+	  break;
+      }
+    while (--n != 0);
+  
+  /* If not enough room in dest, add NUL and traverse rest of src */
+  if (n == 0)
+    {
+      if (dest_size != 0)
+	*d = 0;
+      while (*s++)
+	;
+    }
+  
+  return s - src - 1;  /* count does not include NUL */
+}
+
+/* g_strlcat
+ *
+ * Appends string src to buffer dest (of buffer size dest_size).
+ * At most dest_size-1 characters will be copied.
+ * Unlike strncat, dest_size is the full size of dest, not the space left over.
+ * This function does NOT allocate memory.
+ * This always NUL terminates (unless siz == 0 or there were no NUL characters
+ * in the dest_size characters of dest to start with).
+ * Returns size of attempted result, which is
+ * MIN (dest_size, strlen (original dest)) + strlen (src),
+ * so if retval >= dest_size, truncation occurred.
+ */
+gsize
+g_strlcat (gchar       *dest,
+           const gchar *src,
+           gsize        dest_size)
+{
+  register gchar *d = dest;
+  register const gchar *s = src;
+  register gsize bytes_left = dest_size;
+  gsize dlength;  /* Logically, MIN (strlen (d), dest_size) */
+  
+  g_return_val_if_fail (dest != NULL, 0);
+  g_return_val_if_fail (src  != NULL, 0);
+  
+  /* Find the end of dst and adjust bytes left but don't go past end */
+  while (*d != 0 && bytes_left-- != 0)
+    d++;
+  dlength = d - dest;
+  bytes_left = dest_size - dlength;
+  
+  if (bytes_left == 0)
+    return dlength + strlen (s);
+  
+  while (*s != 0)
+    {
+      if (bytes_left != 1)
+	{
+	  *d++ = *s;
+	  bytes_left--;
+	}
+      s++;
+    }
+  *d = 0;
+  
+  return dlength + (s - src);  /* count does not include NUL */
+}
+#endif /* ! HAVE_STRLCPY */
+
 gchar*
 g_strdown (gchar *string)
 {
   register guchar *s;
-
+  
   g_return_val_if_fail (string != NULL, NULL);
-
-  s = string;
-
+  
+  s = (guchar *) string;
+  
   while (*s)
     {
       *s = tolower (*s);
       s++;
     }
-
-  return string;
+  
+  return (gchar *) string;
 }
 
 gchar*
@@ -826,7 +951,7 @@ g_strup (gchar *string)
 
   g_return_val_if_fail (string != NULL, NULL);
 
-  s = string;
+  s = (guchar *) string;
 
   while (*s)
     {
@@ -834,7 +959,7 @@ g_strup (gchar *string)
       s++;
     }
 
-  return string;
+  return (gchar *) string;
 }
 
 gchar*
@@ -908,8 +1033,9 @@ g_strncasecmp (const gchar *s1,
   g_return_val_if_fail (s1 != NULL, 0);
   g_return_val_if_fail (s2 != NULL, 0);
 
-  while (n-- && *s1 && *s2)
+  while (n && *s1 && *s2)
     {
+      n -= 1;
       /* According to A. Cox, some platforms have islower's that
        * don't work right on non-uppercase
        */
@@ -921,7 +1047,7 @@ g_strncasecmp (const gchar *s1,
     }
 
   if (n)
-    return (((gint)(guchar) *s1) - ((gint)(guchar) *s2));
+    return (((gint) (guchar) *s1) - ((gint) (guchar) *s2));
   else
     return 0;
 #endif
@@ -1026,11 +1152,16 @@ gchar *
 g_strescape (const gchar *source,
 	     const gchar *exceptions)
 {
-  const guchar *p = (guchar *) source;
-  /* Each source byte needs maximally four destination chars (\777) */
-  gchar *dest = g_malloc (strlen (source) * 4 + 1);
-  gchar *q = dest;
+  const guchar *p;
+  gchar *dest;
+  gchar *q;
   guchar excmap[256];
+  
+  g_return_val_if_fail (source != NULL, NULL);
+
+  p = (guchar *) source;
+  /* Each source byte needs maximally four destination chars (\777) */
+  q = dest = g_malloc (strlen (source) * 4 + 1);
 
   memset (excmap, 0, 256);
   if (exceptions)
@@ -1099,224 +1230,6 @@ g_strescape (const gchar *source,
   return dest;
 }
 
-/*
- * g_filename_to_utf8
- *
- * Converts a string which is in the encoding used for file names by
- * the C runtime (usually the same as that used by the operating
- * system) in the current locale into a UTF-8 string.
- */
-
-gchar *
-g_filename_to_utf8 (const gchar *opsysstring)
-{
-#ifdef G_OS_WIN32
-
-  gint i, clen, wclen, first;
-  const gint len = strlen (opsysstring);
-  wchar_t *wcs, wc;
-  gchar *result, *bp;
-  const wchar_t *wcp;
-
-  wcs = g_new (wchar_t, len);
-  wclen = MultiByteToWideChar (CP_ACP, 0, opsysstring, len, wcs, len);
-
-  wcp = wcs;
-  clen = 0;
-  for (i = 0; i < wclen; i++)
-    {
-      wc = *wcp++;
-
-      if (wc < 0x80)
-	clen += 1;
-      else if (wc < 0x800)
-	clen += 2;
-      else if (wc < 0x10000)
-	clen += 3;
-      else if (wc < 0x200000)
-	clen += 4;
-      else if (wc < 0x4000000)
-	clen += 5;
-      else
-	clen += 6;
-    }
-
-  result = g_malloc (clen + 1);
-  
-  wcp = wcs;
-  bp = result;
-  for (i = 0; i < wclen; i++)
-    {
-      wc = *wcp++;
-
-      if (wc < 0x80)
-	{
-	  first = 0;
-	  clen = 1;
-	}
-      else if (wc < 0x800)
-	{
-	  first = 0xc0;
-	  clen = 2;
-	}
-      else if (wc < 0x10000)
-	{
-	  first = 0xe0;
-	  clen = 3;
-	}
-      else if (wc < 0x200000)
-	{
-	  first = 0xf0;
-	  clen = 4;
-	}
-      else if (wc < 0x4000000)
-	{
-	  first = 0xf8;
-	  clen = 5;
-	}
-      else
-	{
-	  first = 0xfc;
-	  clen = 6;
-	}
-      
-      /* Woo-hoo! */
-      switch (clen)
-	{
-	case 6: bp[5] = (wc & 0x3f) | 0x80; wc >>= 6; /* Fall through */
-	case 5: bp[4] = (wc & 0x3f) | 0x80; wc >>= 6; /* Fall through */
-	case 4: bp[3] = (wc & 0x3f) | 0x80; wc >>= 6; /* Fall through */
-	case 3: bp[2] = (wc & 0x3f) | 0x80; wc >>= 6; /* Fall through */
-	case 2: bp[1] = (wc & 0x3f) | 0x80; wc >>= 6; /* Fall through */
-	case 1: bp[0] = wc | first;
-	}
-
-      bp += clen;
-    }
-  *bp = 0;
-
-  g_free (wcs);
-
-  return result;
-
-#else
-
-  return g_strdup (opsysstring);
-
-#endif
-}
-
-/*
- * g_filename_from_utf8
- *
- * The reverse of g_filename_to_utf8.
- */
-
-gchar *
-g_filename_from_utf8 (const gchar *utf8string)
-{
-#ifdef G_OS_WIN32
-
-  gint i, mask, clen, wclen, mblen;
-  const gint len = strlen (utf8string);
-  wchar_t *wcs, *wcp;
-  gchar *result;
-  guchar *cp, *end, c;
-  gint n;
-  
-  /* First convert to wide chars */
-  cp = (guchar *) utf8string;
-  end = cp + len;
-  n = 0;
-  wcs = g_new (wchar_t, len + 1);
-  wcp = wcs;
-  while (cp != end)
-    {
-      mask = 0;
-      c = *cp;
-
-      if (c < 0x80)
-	{
-	  clen = 1;
-	  mask = 0x7f;
-	}
-      else if ((c & 0xe0) == 0xc0)
-	{
-	  clen = 2;
-	  mask = 0x1f;
-	}
-      else if ((c & 0xf0) == 0xe0)
-	{
-	  clen = 3;
-	  mask = 0x0f;
-	}
-      else if ((c & 0xf8) == 0xf0)
-	{
-	  clen = 4;
-	  mask = 0x07;
-	}
-      else if ((c & 0xfc) == 0xf8)
-	{
-	  clen = 5;
-	  mask = 0x03;
-	}
-      else if ((c & 0xfc) == 0xfc)
-	{
-	  clen = 6;
-	  mask = 0x01;
-	}
-      else
-	{
-	  g_free (wcs);
-	  return NULL;
-	}
-
-      if (cp + clen > end)
-	{
-	  g_free (wcs);
-	  return NULL;
-	}
-
-      *wcp = (cp[0] & mask);
-      for (i = 1; i < clen; i++)
-	{
-	  if ((cp[i] & 0xc0) != 0x80)
-	    {
-	      g_free (wcs);
-	      return NULL;
-	    }
-	  *wcp <<= 6;
-	  *wcp |= (cp[i] & 0x3f);
-	}
-
-      cp += clen;
-      wcp++;
-      n++;
-    }
-  if (cp != end)
-    {
-      g_free (wcs);
-      return NULL;
-    }
-
-  /* n is the number of wide chars constructed */
-
-  /* Convert to a string in the current ANSI codepage */
-
-  result = g_new (gchar, 3 * n + 1);
-  mblen = WideCharToMultiByte (CP_ACP, 0, wcs, n, result, 3*n, NULL, NULL);
-  result[mblen] = 0;
-  g_free (wcs);
-
-  return result;
-
-#else
-
-  return g_strdup (utf8string);
-
-#endif
-}
-
 gchar*
 g_strchug (gchar *string)
 {
@@ -1324,10 +1237,10 @@ g_strchug (gchar *string)
 
   g_return_val_if_fail (string != NULL, NULL);
 
-  for (start = string; *start && isspace (*start); start++)
+  for (start = (guchar*) string; *start && isspace (*start); start++)
     ;
 
-  g_memmove (string, start, strlen( start) + 1);
+  g_memmove (string, start, strlen ((gchar *) start) + 1);
 
   return string;
 }
@@ -1356,7 +1269,7 @@ g_strsplit (const gchar *string,
 {
   GSList *string_list = NULL, *slist;
   gchar **str_array, *s;
-  guint i, n = 1;
+  guint n = 1;
 
   g_return_val_if_fail (string != NULL, NULL);
   g_return_val_if_fail (delimiter != NULL, NULL);
@@ -1385,19 +1298,13 @@ g_strsplit (const gchar *string,
 	}
       while (--max_tokens && s);
     }
-  if (*string)
-    {
-      n++;
-      string_list = g_slist_prepend (string_list, g_strdup (string));
-    }
+  string_list = g_slist_prepend (string_list, g_strdup (string));
 
-  str_array = g_new (gchar*, n);
+  str_array = g_new (gchar*, n + 1);
 
-  i = n - 1;
-
-  str_array[i--] = NULL;
+  str_array[n--] = NULL;
   for (slist = string_list; slist; slist = slist->next)
-    str_array[i--] = slist->data;
+    str_array[n--] = slist->data;
 
   g_slist_free (string_list);
 
