@@ -5,23 +5,23 @@
  * Copyright 1998 Sebastian Wilhelmi; University of Karlsruhe
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
 /*
- * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GLib at ftp://ftp.gtk.org/pub/gtk/. 
@@ -51,6 +51,8 @@
 
 gulong g_thread_min_stack_size = 0;
 
+#define G_MUTEX_SIZE (sizeof (mutex_t))
+
 #define HAVE_G_THREAD_IMPL_INIT
 static void 
 g_thread_impl_init()
@@ -108,8 +110,7 @@ g_cond_new_solaris_impl ()
    without error check then!!!!, we might want to change this
    therfore. */
 
-#define G_MICROSEC 1000000
-#define G_NANOSEC 1000000000
+#define G_NSEC_PER_SEC 1000000000
 
 static gboolean
 g_cond_timed_wait_solaris_impl (GCond * cond, 
@@ -130,8 +131,8 @@ g_cond_timed_wait_solaris_impl (GCond * cond,
     }
 
   end_time.tv_sec = abs_time->tv_sec;
-  end_time.tv_nsec = abs_time->tv_usec * (G_NANOSEC / G_MICROSEC);
-  g_assert (end_time.tv_nsec < G_NANOSEC);
+  end_time.tv_nsec = abs_time->tv_usec * (G_NSEC_PER_SEC / G_USEC_PER_SEC);
+  g_assert (end_time.tv_nsec < G_NSEC_PER_SEC);
   result = cond_timedwait ((cond_t *) cond, (mutex_t *) entered_mutex,
 			   &end_time);
   timed_out = (result == ETIME);
@@ -195,18 +196,28 @@ g_thread_create_solaris_impl (GThreadFunc thread_func,
 			      gboolean joinable,
 			      gboolean bound,
 			      GThreadPriority priority,
-			      gpointer thread)
+			      gpointer thread,
+			      GError **error)
 {     
   long flags = (bound ? THR_BOUND : 0) | (joinable ? 0: THR_DETACHED);
+  gint ret;
   
   g_return_if_fail (thread_func);
   
   stack_size = MAX (g_thread_min_stack_size, stack_size);
   
-  solaris_check_for_error (thr_create (NULL, stack_size,  
-				       (void* (*)(void*))thread_func,
-				       arg, flags, thread));
+  ret = thr_create (NULL, stack_size, (void* (*)(void*))thread_func,
+		    arg, flags, thread);
+
+  if (ret == EAGAIN)
+    {
+      g_set_error (error, G_THREAD_ERROR, G_THREAD_ERROR_AGAIN, 
+		   "Error creating thread: %s", g_strerror (ret));
+      return;
+    }
   
+  solaris_check_for_error (ret);
+
   g_thread_set_priority_solaris_impl (thread, priority);
 }
 
