@@ -3,23 +3,23 @@
  * Copyright (C) 1998-1999  Tor Lillqvist
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
 /*
- * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GLib at ftp://ftp.gtk.org/pub/gtk/. 
@@ -130,7 +130,7 @@ g_win32_opendir (const char *dirname)
 struct dirent*
 g_win32_readdir (DIR *dir)
 {
-  static struct dirent result;
+  gchar *basename;
 
   g_return_val_if_fail (dir != NULL, NULL);
 
@@ -153,9 +153,14 @@ g_win32_readdir (DIR *dir)
 	    }
 	}
     }
-  strcpy (result.d_name, g_basename (((LPWIN32_FIND_DATA) dir->find_file_data)->cFileName));
+  
+  basename = g_path_get_basename (((LPWIN32_FIND_DATA) dir->find_file_data)->cFileName);
+
+  strcpy (dir->readdir_result.d_name, basename);
+
+  g_free (basename);
       
-  return &result;
+  return &dir->readdir_result;
 }
 
 void
@@ -593,7 +598,7 @@ g_win32_getlocale (void)
 				 */
       switch (sub)
 	{
-	case SUBLANG_SERBIAN_LATIN: l = "hr"; break;
+	case SUBLANG_SERBIAN_LATIN: l = "sp"; break;
 	case SUBLANG_SERBIAN_CYRILLIC: l = "sr"; break;
 	default: l = "hr";	/* ??? */
 	}
@@ -800,11 +805,13 @@ g_win32_error_message (gint error)
   gchar *retval;
   int nbytes;
 
-  nbytes = FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER
-			  |FORMAT_MESSAGE_IGNORE_INSERTS
-			  |FORMAT_MESSAGE_FROM_SYSTEM,
-			  NULL, error, 0,
-			  (LPTSTR) &msg, 0, NULL);
+  FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER
+		 |FORMAT_MESSAGE_IGNORE_INSERTS
+		 |FORMAT_MESSAGE_FROM_SYSTEM,
+		 NULL, error, 0,
+		 (LPTSTR) &msg, 0, NULL);
+  nbytes = strlen (msg);
+
   if (nbytes > 2 && msg[nbytes-1] == '\n' && msg[nbytes-2] == '\r')
     msg[nbytes-2] = '\0';
   
