@@ -2,34 +2,46 @@
  * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
 /*
- * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GLib at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include "config.h"
+
 #undef G_LOG_DOMAIN
 
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
+
 #include "glib.h"
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef G_OS_WIN32
+#include <io.h>			/* For read(), write() etc */
+#endif
+
 int array[10000];
 gboolean failed = FALSE;
 
@@ -238,9 +250,9 @@ my_hash (gconstpointer key)
   return (guint) *((const gint*) key);
 }
 
-static gint
-my_hash_compare (gconstpointer a,
-		 gconstpointer b)
+static gboolean
+my_hash_equal (gconstpointer a,
+	       gconstpointer b)
 {
   return *((const gint*) a) == *((const gint*) b);
 }
@@ -349,6 +361,12 @@ main (int   argc,
   guint64 gu64t1 = G_GINT64_CONSTANT(0x1d636b02300a7aa7U),
 	  gu64t2 = G_GINT64_CONSTANT(0xa77a0a30026b631dU);
 #endif
+  const char hello[] = "Hello, World";
+  const int hellolen = sizeof (hello) - 1;
+  int fd;
+  char template[10];
+  GError *error;
+  char *name_used;
 
   g_print ("TestGLib v%u.%u.%u (i:%u b:%u)\n",
 	   glib_major_version,
@@ -372,18 +390,19 @@ main (int   argc,
   TEST (NULL, sizeof (gint16) == 2);
   g_print ("\nchecking size of gint32: %d", (int)sizeof (gint32));
   TEST (NULL, sizeof (gint32) == 4);
+  g_print ("\nchecking size of gsize: %d", (int)sizeof (gsize));
 #ifdef	G_HAVE_GINT64
   g_print ("\nchecking size of gint64: %d", (int)sizeof (gint64));
   TEST (NULL, sizeof (gint64) == 8);
 #endif	/* G_HAVE_GINT64 */
   g_print ("\n");
 
-  g_print ("checking g_dirname()...");
+  g_print ("checking g_path_get_dirname()...");
   for (i = 0; i < n_dirname_checks; i++)
     {
       gchar *dirname;
 
-      dirname = g_dirname (dirname_checks[i].filename);
+      dirname = g_path_get_dirname (dirname_checks[i].filename);
       if (strcmp (dirname, dirname_checks[i].dirname) != 0)
 	{
 	  g_print ("\nfailed for \"%s\"==\"%s\" (returned: \"%s\")\n",
@@ -616,7 +635,7 @@ main (int   argc,
 
   g_print ("checking hash tables...");
 
-  hash_table = g_hash_table_new (my_hash, my_hash_compare);
+  hash_table = g_hash_table_new (my_hash, my_hash_equal);
   for (i = 0; i < 10000; i++)
     {
       array[i] = i;
@@ -827,6 +846,13 @@ main (int   argc,
   g_string_free (string1, TRUE);
   g_string_free (string2, TRUE);
   
+  g_print ("test positional printf formats (not supported): ");
+  string = g_strdup_printf ("%.*s%s", 5, "a", "b");
+  tmp_string = g_strdup_printf ("%2$*1$s", 5, "c");
+  g_print ("%s%s\n", string, tmp_string);
+  g_free (tmp_string);
+  g_free (string);
+
   g_print ("checking timers...\n");
   
   timer = g_timer_new ();
@@ -879,9 +905,66 @@ main (int   argc,
   g_assert(strcmp(string, GLIB_TEST_STRING GLIB_TEST_STRING
   			  GLIB_TEST_STRING) == 0);
   g_free(string);
-  
   g_print ("ok\n");
+  
 
+  g_print("checking g_strlcpy/g_strlcat...");
+  /* The following is a torture test for strlcpy/strlcat, with lots of
+   * checking; normal users wouldn't use them this way!
+   */
+  string = g_malloc (6);
+  *(string + 5) = 'Z'; /* guard value, shouldn't change during test */
+  *string = 'q';
+  g_assert (g_strlcpy(string, "" , 5) == 0);
+  g_assert ( *string == '\0' );
+  *string = 'q';
+  g_assert (g_strlcpy(string, "abc" , 5) == 3);
+  g_assert ( *(string + 3) == '\0' );
+  g_assert (g_str_equal(string, "abc"));
+  g_assert (g_strlcpy(string, "abcd" , 5) == 4);
+  g_assert ( *(string + 4) == '\0' );
+  g_assert ( *(string + 5) == 'Z' );
+  g_assert (g_str_equal(string, "abcd"));
+  g_assert (g_strlcpy(string, "abcde" , 5) == 5);
+  g_assert ( *(string + 4) == '\0' );
+  g_assert ( *(string + 5) == 'Z' );
+  g_assert (g_str_equal(string, "abcd"));
+  g_assert (g_strlcpy(string, "abcdef" , 5) == 6);
+  g_assert ( *(string + 4) == '\0' );
+  g_assert ( *(string + 5) == 'Z' );
+  g_assert (g_str_equal(string, "abcd"));
+  *string = 'Y';
+  *(string + 1)= '\0';
+  g_assert (g_strlcpy(string, "Hello" , 0) == 5);
+  g_assert (*string == 'Y');
+  *string = '\0';
+  g_assert (g_strlcat(string, "123" , 5) == 3);
+  g_assert ( *(string + 3) == '\0' );
+  g_assert (g_str_equal(string, "123"));
+  g_assert (g_strlcat(string, "" , 5) == 3);
+  g_assert ( *(string + 3) == '\0' );
+  g_assert (g_str_equal(string, "123"));
+  g_assert (g_strlcat(string, "4", 5) == 4);
+  g_assert (g_str_equal(string, "1234"));
+  g_assert (g_strlcat(string, "5", 5) == 5);
+  g_assert ( *(string + 4) == '\0' );
+  g_assert (g_str_equal(string, "1234"));
+  g_assert ( *(string + 5) == 'Z' );
+  *string = 'Y';
+  *(string + 1)= '\0';
+  g_assert (g_strlcat(string, "123" , 0) == 3);
+  g_assert (*string == 'Y');
+  
+  /* A few more tests, demonstrating more "normal" use  */
+  g_assert (g_strlcpy(string, "hi", 5) == 2);
+  g_assert (g_str_equal(string, "hi"));
+  g_assert (g_strlcat(string, "t", 5) == 3);
+  g_assert (g_str_equal(string, "hit"));
+  g_free(string);
+
+  g_print ("ok\n");
+  
+  
   g_print ("checking g_strdup_printf...");
   string = g_strdup_printf ("%05d %-5s", 21, "test");
   g_assert (string != NULL);
@@ -1020,6 +1103,71 @@ main (int   argc,
   g_print ("current locale: %s\n", g_win32_getlocale ());
 #endif
 
+  g_print ("checking file functions...\n");
+
+  strcpy (template, "foobar");
+  fd = g_mkstemp (template);
+  if (fd != -1)
+    g_print ("g_mkstemp works even if template doesn't end in XXXXXX\n");
+  close (fd);
+  strcpy (template, "fooXXXXXX");
+  fd = g_mkstemp (template);
+  if (fd == -1)
+    g_print ("g_mkstemp didn't work for template %s\n", template);
+  i = write (fd, hello, hellolen);
+  if (i == -1)
+    g_print ("write() failed: %s\n", g_strerror (errno));
+  else if (i != hellolen)
+    g_print ("write() should have written %d bytes, wrote %d\n", hellolen, i);
+
+  lseek (fd, 0, 0);
+  i = read (fd, chars, sizeof (chars));
+  if (i == -1)
+    g_print ("read() failed: %s\n", g_strerror (errno));
+  else if (i != hellolen)
+    g_print ("read() should have read %d bytes, got %d\n", hellolen, i);
+
+  chars[i] = 0;
+  if (strcmp (chars, hello) != 0)
+    g_print ("wrote '%s', but got '%s'\n", hello, chars);
+
+  close (fd);
+  remove (template);
+
+  strcpy (template, "zap" G_DIR_SEPARATOR_S "barXXXXXX");
+  fd = g_file_open_tmp (template, &name_used, &error);
+  if (fd != -1)
+    g_print ("g_file_open_tmp works even if template contains '%s'\n",
+	     G_DIR_SEPARATOR_S);
+  else
+    g_print ("g_file_open_tmp correctly returns error: %s\n",
+	     error->message);
+  close (fd);
+  g_clear_error (&error);
+
+  strcpy (template, "zapXXXXXX");
+  fd = g_file_open_tmp (template, &name_used, &error);
+  if (fd == -1)
+    g_print ("g_file_open_tmp didn't work for template '%s': %s\n",
+	     template, error->message);
+  else
+    g_print ("g_file_open_tmp for template '%s' used name '%s'\n",
+	     template, name_used);
+  close (fd);
+  g_clear_error (&error);
+  remove (name_used);
+
+  fd = g_file_open_tmp (NULL, &name_used, &error);
+  if (fd == -1)
+    g_print ("g_file_open_tmp didn't work for a NULL template: %s\n",
+	     error->message);
+  else
+    g_print ("g_file_open_tmp for NULL template used name '%s'\n",
+	     name_used);
+  close (fd);
+  g_clear_error (&error);
+  remove (name_used);
+  
   return 0;
 }
 
